/**
 * Copyright (C) cedarsoft GmbH.
 *
 * Licensed under the GNU General Public License version 3 (the "License")
 * with Classpath Exception; you may not use this file except in compliance
 * with the License. You may obtain a copy of the License at
 *
 *         http://www.cedarsoft.org/gpl3ce
 *         (GPL 3 with Classpath Exception)
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 3 only, as
 * published by the Free Software Foundation. cedarsoft GmbH designates this
 * particular file as subject to the "Classpath" exception as provided
 * by cedarsoft GmbH in the LICENSE file that accompanied this code.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 3 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 3 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact cedarsoft GmbH, 72810 Gomaringen, Germany,
 * or visit www.cedarsoft.com if you need additional information or
 * have any questions.
 */

package com.cedarsoft.business.contact;

import com.cedarsoft.business.contact.communication.CommunicationChannel;
import com.cedarsoft.business.contact.communication.Email;
import com.cedarsoft.business.contact.communication.FaxNumber;
import com.cedarsoft.business.contact.communication.PhoneNumber;
import com.cedarsoft.event.ClusteredPropertyChangeSupport;
import com.cedarsoft.history.ElementVisitor;
import com.cedarsoft.history.ElementsListener;
import org.jetbrains.annotations.NonNls;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.beans.PropertyChangeListener;
import java.util.List;
import java.util.concurrent.locks.ReentrantReadWriteLock;

/**
 * A contact
 */
public class Contact {
  @NotNull
  @NonNls
  public static final String PROPERTY_ADDRESSES = "addresses";
  /**
   * Property for the first address
   */
  @NotNull
  @NonNls
  public static final String PROPERTY_ADDRESS = "address";
  @NotNull
  @NonNls
  public static final String PROPERTY_ID = "id";
  @NotNull
  @NonNls
  public static final String PROPERTY_TYPE = "type";
  @NotNull
  @NonNls
  public static final String PROPERTY_OTHER_COMMUNICATION_CHANNELS = "otherCommunicationChannels";
  @NotNull
  @NonNls
  public static final String PROPERTY_BANK_DETAILS = "bankDetails";


  private Long id;

  @NotNull
  protected final ReentrantReadWriteLock lock = new ReentrantReadWriteLock();
  @NotNull
  protected final ClusteredPropertyChangeSupport pcs = new ClusteredPropertyChangeSupport( this );
  @NotNull
  private final BankDetailsList bankDetails = new BankDetailsList();
  @NotNull
  private final CommunicationChannels otherCommunicationChannels = new CommunicationChannels();
  @NotNull
  private final Addresses addresses = new Addresses();

  public void addBankDetails( @NotNull BankDetails additionalBankDetails ) {
    bankDetails.addElement( additionalBankDetails );
  }

  @SuppressWarnings( {"ParameterHidesMemberVariable"} )
  public void removeBankDetails( BankDetails bankDetails ) {
    this.bankDetails.removeEntry( bankDetails );
  }

  public void addCommunicationChannel( @NotNull CommunicationChannel communicationChannel ) {
    otherCommunicationChannels.addElement( communicationChannel );
  }

  public void removeCommunicationChannel( @NotNull CommunicationChannel communicationChannel ) {
    otherCommunicationChannels.removeEntry( communicationChannel );
  }

  @Deprecated
  public void removeAddress( @NotNull BaseAddress address ) {
    addresses.removeEntry( address );
  }

  @NotNull
  public Addresses getAddresses() {
    return addresses;
  }

  @NotNull
  public CommunicationChannels getOtherCommunicationChannels() {
    return otherCommunicationChannels;
  }

  public void addAddress( @NotNull BaseAddress address ) {
    addresses.addElement( address );
  }

  @NotNull
  public BaseAddress getAddress() {
    addresses.getLock().writeLock().lock();
    try {
      if ( addresses.isEmpty() ) {
        Address address = new Address();
        address.setActive( true );
        addAddress( address );
      }
      return addresses.getElements().get( 0 );
    } finally {
      addresses.getLock().writeLock().unlock();
    }
  }

  @NotNull
  public FaxNumber getFax() throws IllegalStateException {
    return otherCommunicationChannels.getFirstActiveFax();
  }

  @NotNull
  public Email getMail() throws IllegalStateException {
    return otherCommunicationChannels.getFirstActiveMail();
  }

  @NotNull
  public PhoneNumber getPhoneNumber() throws IllegalStateException {
    return otherCommunicationChannels.getFirstActivePhoneNumber();
  }

  /**
   * Returns all communication channels
   *
   * @return the communication channels
   */
  @NotNull
  public List<? extends CommunicationChannel> getCommunicationChannels() {
    return getOtherCommunicationChannels().getElements();
  }

  @NotNull
  public BankDetailsList getBankDetails() {
    return bankDetails;
  }

  @Nullable
  public BaseAddress getFirstActiveAddress() {
    return addresses.findFirstElementNullable( new ElementVisitor<BaseAddress>( "first active address" ) {
      @Override
      public boolean fits( @NotNull BaseAddress element ) {
        return element.isActive();
      }
    } );
  }

  public void addAddressListener( @NotNull ElementsListener<BaseAddress> listener ) {
    addresses.addElementListener( listener );
  }

  @Nullable
  public Long getId() {
    return id;
  }

  public void removePropertyChangeListener( @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.removePropertyChangeListener( listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void removePropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.removePropertyChangeListener( propertyName, listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull PropertyChangeListener listener, boolean isTransient ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( listener, isTransient );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( propertyName, listener );
    } finally {
      lock.writeLock().unlock();
    }
  }

  public void addPropertyChangeListener( @NotNull @NonNls String propertyName, @NotNull PropertyChangeListener listener, boolean isTransient ) {
    lock.writeLock().lock();
    try {
      pcs.addPropertyChangeListener( propertyName, listener, isTransient );
    } finally {
      lock.writeLock().unlock();
    }
  }
}
